@@ -9,10 +9,10 @@
  */
 /*
 
-  knetfilter v 2.2.0
+  knetfilter v 3.0.2
 
-  Copyright (C) 2001 Luigi Genoni
-                     venom@DarkStar.sns.it
+  Copyright (C) 2002 Luigi Genoni
+                     venom@sns.it
 */
 
 #include <stdlib.h>
