@@ -1,9 +1,9 @@
   /*
 
-  knetfilter v 2.2.0
+  knetfilter v. 3.1.0
 
-  Copyright (C) 2001 Luigi Genoni
-  		     venom@DarkStar.sns.it
+  Copyright (C) 2002 Luigi Genoni
+  		     venom@sns.it
   */
   
 
@@ -25,7 +25,7 @@
 
 void knetfilter::startmangleRules() {
 
-    QString firstmanglenet, firstmanglenetmask, firstmangleport, firstmanglenetwork, secondmanglenet, secondmanglenetmask, secondmangleport, secondmanglenetwork, manglesyn, manglemark, manglelogname, manglerulepos; 
+    QString firstmanglenet, firstmanglenetmask, firstmangleport, firstmanglenetwork, secondmanglenet, secondmanglenetmask, secondmangleport, secondmanglenetwork, manglesyn, manglemark, mangledscp, manglelogname, manglerulepos, mangleulogname, mangleulogNlgroup, mangleulogCprange; 
 
     // let's check everything
 
@@ -37,7 +37,11 @@ void knetfilter::startmangleRules() {
     secondmangleport.sprintf((const char *)(secondmanglePort->text()));
     manglerulepos.sprintf((const char *)(addmangleRuleNumber->text()));
     manglemark.sprintf((const char *)(mangleMark->text()));
+    mangledscp.sprintf((const char *)(mangleDscp->text()));
     manglelogname.sprintf((const char*)(logName->text()));
+    mangleulogname.sprintf((const char*)(ulogName->text()));
+    mangleulogNlgroup.sprintf((const char*)(ulogNlgroup->text()));
+    mangleulogCprange.sprintf((const char*)(ulogCprange->text()));
 
     if(firstmanglenetmask.length()==0)
       {
@@ -136,6 +140,54 @@ void knetfilter::startmangleRules() {
 	  }
       };
 
+    if (strcmp(mangleactionChoice,"DSCP")==0)
+      { 
+	if(mangledscp.length()==0)
+	  {
+	    QMessageBox::critical(0, "Error",
+				  "You have to supply a DSCP esadecimal value to set.\n");
+	  }
+	else
+	  {
+	    if (strcmp(mangleprotocolChoice,"icmp")==0)
+	      {
+		if(firstmangleport.length()==0) // fp=n
+		  {
+		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		  }
+		else
+		  {
+		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		  }
+	      }	
+	    else
+	      {
+		if(firstmangleport.length()==0) // fp=n
+		  {	
+		    if(secondmangleport.length()==0) // fp=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		      }
+		    else // fp=n sp!=n
+		      { 
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		      }
+		  }
+		else //fp!=n
+		  {	    
+		    if(secondmangleport.length()==0) // fp!=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		      }
+		    else // fp!=n sp!=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		      }	
+		  }	
+	      }
+	  }
+      };
+
     if (strcmp(mangleactionChoice,"MARK")==0)
       {
 	if (manglemark.length()==0)
@@ -184,7 +236,7 @@ void knetfilter::startmangleRules() {
 	  }
       };
 
-    if (strcmp(mangleactionChoice,"LOG")==0)
+    if (strcmp(mangleactionChoice,"LOG")==0) 
       {
 	if (strcmp(mangleprotocolChoice,"icmp")==0)
 	  {
@@ -224,6 +276,54 @@ void knetfilter::startmangleRules() {
 	  }	
       };
 
+    if (strcmp(mangleactionChoice,"ULOG")==0) 
+      {
+	if (strcmp(mangleulogNlgroup,"0")==0) 
+	  {
+	    QMessageBox::critical(0, "Error",
+				  "NetLink group should be beetwen 1 and 32\n");
+	  }
+	else	  
+	  {
+	    if (strcmp(mangleprotocolChoice,"icmp")==0)
+	      {
+		if(firstmangleport.length()==0) // fp=n
+		  {
+		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+		  }
+		else // fp!=n
+		  {
+		    addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+		  }
+	      }
+	    else
+	      {
+		if(firstmangleport.length()==0) // fp=n
+		  {	
+		    if(secondmangleport.length()==0) // fp=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-prefix" << mangleulogname;  
+		      }
+		    else // fp=n sp!=n
+		      { 
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "ULOG" << "--log-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+		      }
+		  }
+		else // fp!=n
+		  {	    
+		    if(secondmangleport.length()==0) // fp!=n sp=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+		      }
+		    else // fp!=n sp!=n
+		      {
+			addmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+		      }	
+		  }	
+	      }	
+	  }
+      };
+
     bool success=addmangle.start();
     if(success)
       statusbar->message(i18n("Running..."), 2000);
@@ -242,8 +342,12 @@ void knetfilter::startmangleRules() {
       secondmangleNetMask->clear();
       secondmanglePort->clear();
       mangleMark->clear();
+      mangleDscp->clear();
       addmangleRuleNumber->clear();
       logName->setText("mangleLog");
+      ulogNlgroup->setText("1");
+      ulogCprange->setText("0");
+      ulogName->setText("mangleULog");
     } else
       {
 	statusbar->message(i18n("iptables died"), 2000);
@@ -261,7 +365,7 @@ void knetfilter::startmangleRules() {
 
 void knetfilter::stopmangleRules(){
 
-    QString firstmanglenet, firstmanglenetmask, firstmangleport, firstmanglenetwork, secondmanglenet, secondmanglenetmask, secondmangleport, secondmanglenetwork, manglesyn, manglemark, manglelogname, manglerulepos; 
+    QString firstmanglenet, firstmanglenetmask, firstmangleport, firstmanglenetwork, secondmanglenet, secondmanglenetmask, secondmangleport, secondmanglenetwork, manglesyn, manglemark, mangledscp, manglelogname, manglerulepos, mangleulogname, mangleulogNlgroup, mangleulogCprange; 
     // let's check everything
 
     firstmanglenet.sprintf((const char *)(firstmangleNet->text()));
@@ -272,7 +376,11 @@ void knetfilter::stopmangleRules(){
     secondmangleport.sprintf((const char *)(secondmanglePort->text()));
     manglerulepos.sprintf((const char *)(addmangleRuleNumber->text()));
     manglemark.sprintf((const char *)(mangleMark->text()));
+    mangledscp.sprintf((const char *)(mangleDscp->text()));
     manglelogname.sprintf((const char*)(logName->text()));
+    mangleulogname.sprintf((const char*)(ulogName->text()));
+    mangleulogNlgroup.sprintf((const char*)(ulogNlgroup->text()));
+    mangleulogCprange.sprintf((const char*)(ulogCprange->text()));
 
     if(firstmanglenetmask.length()==0)
       {
@@ -369,6 +477,54 @@ void knetfilter::stopmangleRules(){
 	      }	    
 	  };
 
+	if (strcmp(mangleactionChoice,"DSCP")==0)
+	  { 
+	    if(mangledscp.length()==0)
+	      {
+		QMessageBox::critical(0, "Error",
+				      "You have to supply a DSCP esadecimal value to set.\n");
+	      }
+	    else
+	      {
+		if (strcmp(mangleprotocolChoice,"icmp")==0)
+		  {
+		    if(firstmangleport.length()==0) // fp=n
+		      {
+			delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		      }
+		    else
+		      {
+			delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+		      }
+		  }	
+		else
+		  {
+		    if(firstmangleport.length()==0) // fp=n
+		      {	
+			if(secondmangleport.length()==0) // fp=n sp=n
+			  {
+			    delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }
+			else // fp=n sp!=n
+			  { 
+			    delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }
+		      }
+		    else //fp!=n
+		      {	    
+			if(secondmangleport.length()==0) // fp!=n sp=n
+			  {
+			    delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }
+			else // fp!=n sp!=n
+			  {
+			    delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << manglesyn << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "DSCP" << "--set-dscp" << mangledscp;
+			  }	
+		      }	
+		  }
+	      }
+	  };
+	
 	if (strcmp(mangleactionChoice,"MARK")==0)
 	  {
 	    if (manglemark.length()==0)
@@ -458,6 +614,54 @@ void knetfilter::stopmangleRules(){
 	  }
       };
 
+    if (strcmp(mangleactionChoice,"ULOG")==0)
+      {
+	if (strcmp(mangleulogNlgroup,"0")==0) 
+	  {
+	    QMessageBox::critical(0, "Error",
+				  "NetLink group should be beetwen 1 and 32\n");          
+	  }
+	else      	      
+	  {
+	    if (strcmp(mangleprotocolChoice,"icmp")==0)
+	      {
+		if(firstmangleport.length()==0) // fp=n
+		  {
+		    delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;  
+		  }
+		else // fp!=n
+		  {
+		    delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "--icmp-type" << firstmangleport << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;
+		  }
+	      }
+	    else
+	      {
+		if(firstmangleport.length()==0) // fp=n
+		  {	
+		    if(secondmangleport.length()==0) // fp=n sp=n
+		      {
+			delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;
+		      }
+		    else // fp=n sp!=n
+		      { 
+			delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;
+		      }
+		  }
+		else // fp!=n
+		  {	    
+		    if(secondmangleport.length()==0) // fp!=n sp=n
+		      {
+			delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;
+		      }
+		    else // fp!=n sp!=n
+		      {
+			delmangle << "iptables" << "-t" << "mangle" << mangleruleaction << manglechainChoice << manglerulepos << "-p" << mangleprotocolChoice << "-s" << firstmanglenetwork << "--source-port" << firstmangleport << "-d" << secondmanglenetwork << "--destination-port" << secondmangleport << "-j" << "ULOG" << "--ulog-nlgroup" << mangleulogNlgroup << "--ulog-cprange" << mangleulogCprange << "--ulog-prefix" << mangleulogname;
+		      }	
+		  }	
+	      }		    
+	  }
+      };
+
     bool success=delmangle.start();
     if(success)
       statusbar->message(i18n("Running..."), 2000);
@@ -476,6 +680,7 @@ void knetfilter::stopmangleRules(){
       secondmangleNetMask->clear();
       secondmanglePort->clear();
       mangleMark->clear();
+      mangleDscp->clear();
       addmangleRuleNumber->clear();
       logName->setText("mangleLog");
     } else
@@ -500,12 +705,12 @@ void knetfilter::mangleRules() {
       setupConnect();
       
       mangleList = new KNFWidget();
-      mangleList->resize(640,380);
+      mangleList->resize(660,380);
       mangleList->setCaption("Set Mangle Policy");
 
       QBoxLayout *top = new QBoxLayout(mangleList,  QBoxLayout::LeftToRight, 10);
-      QGridLayout *grid = new QGridLayout(10,10);
-      top->addLayout(grid,10);
+      QGridLayout *grid = new QGridLayout(11,11);
+      top->addLayout(grid,11);
 
       QLabel *labelChainmangle = new QLabel(mangleList);
       labelChainmangle->setText("Action:");
@@ -513,7 +718,10 @@ void knetfilter::mangleRules() {
       
       manglechain = new QComboBox( FALSE, mangleList, "Action");
       manglechain->insertItem("PREROUTING");
+      manglechain->insertItem("INPUT");
+      manglechain->insertItem("FORWARD");
       manglechain->insertItem("OUTPUT");
+      manglechain->insertItem("POSTROUTING");
       manglechain->setFixedHeight(manglechain->sizeHint().height());
       connect(manglechain,SIGNAL(activated(int)), SLOT(manglechainActivated(int)));
 
@@ -549,9 +757,9 @@ void knetfilter::mangleRules() {
       QLabel *labelfirstmanglePort = new QLabel(mangleList);
       labelfirstmanglePort->setText("port (ICMP type):");
 
-      grid->addMultiCellWidget(labelfirstmangleNet,2,2,0,2);
+      grid->addMultiCellWidget(labelfirstmangleNet,2,2,0,1);
       grid->addMultiCellWidget(labelfirstmangleNetMask,2,2,4,5);
-      grid->addMultiCellWidget(labelfirstmanglePort,2,2,7,8);
+      grid->addMultiCellWidget(labelfirstmanglePort,2,2,8,8);
 
       firstmangleNet = new QLineEdit(mangleList);
       firstmangleNetMask = new QLineEdit(mangleList);
@@ -562,7 +770,7 @@ void knetfilter::mangleRules() {
       firstmanglePort->setFixedHeight(firstmanglePort->sizeHint().height());
 
       grid->addMultiCellWidget(firstmangleNet,2,2,2,3);
-      grid->addMultiCellWidget(firstmangleNetMask,2,2,5,6);
+      grid->addMultiCellWidget(firstmangleNetMask,2,2,6,7);
       grid->addMultiCellWidget(firstmanglePort,2,2,9,9);
 
       QLabel *labelsecondmangleNet = new QLabel(mangleList);
@@ -574,9 +782,9 @@ void knetfilter::mangleRules() {
       QLabel *labelsecondmanglePort = new QLabel(mangleList);
       labelsecondmanglePort->setText("port:");
 
-      grid->addMultiCellWidget(labelsecondmangleNet,3,3,0,2);
+      grid->addMultiCellWidget(labelsecondmangleNet,3,3,0,1);
       grid->addMultiCellWidget(labelsecondmangleNetMask,3,3,4,5);
-      grid->addMultiCellWidget(labelsecondmanglePort,3,3,7,8);
+      grid->addMultiCellWidget(labelsecondmanglePort,3,3,8,8);
 
       secondmangleNet = new QLineEdit(mangleList);
       secondmangleNetMask = new QLineEdit(mangleList);
@@ -587,7 +795,7 @@ void knetfilter::mangleRules() {
       secondmanglePort->setFixedHeight(secondmanglePort->sizeHint().height());
 
       grid->addMultiCellWidget(secondmangleNet,3,3,2,3);
-      grid->addMultiCellWidget(secondmangleNetMask,3,3,5,6);
+      grid->addMultiCellWidget(secondmangleNetMask,3,3,6,7);
       grid->addMultiCellWidget(secondmanglePort,3,3,9,9);
 
 
@@ -597,8 +805,10 @@ void knetfilter::mangleRules() {
 
       mangleaction = new QComboBox (FALSE, mangleList, "Flag to modify");
       mangleaction->insertItem("TOS");
+      mangleaction->insertItem("DSCP");
       mangleaction->insertItem("MARK");
       mangleaction->insertItem("LOG");
+      mangleaction->insertItem("ULOG");
       mangleaction->setFixedHeight(mangleaction->sizeHint().height());
       connect(mangleaction,SIGNAL(activated(int)), SLOT(mangleactionActivated(int))); 
       grid->addMultiCellWidget(mangleaction,4,4,2,3);
@@ -623,7 +833,7 @@ void knetfilter::mangleRules() {
 
       QLabel *labelmangleMark = new QLabel(mangleList);
       labelmangleMark->setText("MARK:");
-      grid->addMultiCellWidget(labelmangleMark,6,6,5,6);
+      grid->addMultiCellWidget(labelmangleMark,6,6,5,5);
 
       mangleMark = new QLineEdit(mangleList);
 
@@ -631,6 +841,16 @@ void knetfilter::mangleRules() {
 
       grid->addMultiCellWidget(mangleMark,6,6,6,7);
 
+      QLabel *labelmangleDscp = new QLabel(mangleList);
+      labelmangleDscp->setText("DSCP:");
+      grid->addMultiCellWidget(labelmangleDscp,6,6,7,7);
+
+      mangleDscp = new QLineEdit(mangleList);
+
+      mangleDscp->setFixedHeight(mangleDscp->sizeHint().height());
+
+      grid->addMultiCellWidget(mangleDscp,6,6,8,8);
+
       QLabel *labelLog = new QLabel(mangleList);
       labelLog->setText("LOG rules:  level:");
       grid->addMultiCellWidget(labelLog,7,7,0,1);
@@ -685,26 +905,59 @@ void knetfilter::mangleRules() {
       grid->addMultiCellWidget(manglelog3,8,8,8,9);
       
 	
+      QLabel *labelulogNlgroup = new QLabel(mangleList);
+      labelulogNlgroup->setText("ULOG NLgroup (1/32):");
+      grid->addMultiCellWidget(labelulogNlgroup,9,9,0,2);
+
+      ulogNlgroup = new QLineEdit(mangleList);
+      ulogNlgroup->setText("1");
+      grid->addMultiCellWidget(ulogNlgroup,9,9,3,3);
+
+      QLabel *labelulogCprange = new QLabel(mangleList);
+      labelulogCprange->setText("copy range:");
+      grid->addMultiCellWidget(labelulogCprange,9,9,4,5);
+
+      ulogCprange = new QLineEdit(mangleList);
+      ulogCprange->setText("0");
+      grid->addMultiCellWidget(ulogCprange,9,9,6,6);
+
+      //      QLabel *labelulogQthreshold = new QLabel(mangleList);
+      //      labelulogQthreshold->setText("queue threshold:");
+      //      grid->addMultiCellWidget(labelulogQthreshold,9,9,4,4);
+
+      //      ulogQthreshold = new QLineEdit(mangleList);
+      //      ulogQthreshold->setText("1");
+      //      grid->addMultiCellWidget(ulogQthreshold,9,9,5,5);
+
+      QLabel *labelulogName = new QLabel(mangleList);
+      labelulogName->setText("prefix:");
+      grid->addMultiCellWidget(labelulogName,9,9,7,7);
+
+      ulogName = new QLineEdit(mangleList);
+      ulogName->setText("mangleULog");
+      grid->addMultiCellWidget(ulogName,9,9,8,9);
+
+
       QPushButton *btnRules = new QPushButton(mangleList);
       btnRules->setText("Commit");
       btnRules->setFixedHeight(btnRules->sizeHint().height());
       connect(btnRules,SIGNAL(clicked()), this, SLOT(startmangleRules()));
-      grid->addWidget(btnRules,9,1);
+      grid->addWidget(btnRules,10,1);
 
       QPushButton *btnDelete = new QPushButton(mangleList);
       btnDelete->setText("Delete");
       btnDelete->setFixedHeight(btnDelete->sizeHint().height());
       connect(btnDelete,SIGNAL(clicked()), this, SLOT(stopmangleRules()));
-      grid->addWidget(btnDelete,9,5);
+      grid->addWidget(btnDelete,10,5);
 
       QPushButton *btnClose = new QPushButton(mangleList);
       btnClose->setText("Close");
       btnClose->setFixedHeight(btnClose->sizeHint().height());
       connect(btnClose,SIGNAL(clicked()), this, SLOT(closemangleRules()));
-      grid->addWidget(btnClose,9,7);
+      grid->addWidget(btnClose,10,7);
 
       mangleList->show();
-      setView(mangleList);
+      setCentralWidget(mangleList);
     }
   else
     {
@@ -727,8 +980,17 @@ void knetfilter::manglechainActivated(int index) {
     manglechainChoice = "PREROUTING";
     break;
   case 1:
+    manglechainChoice = "INPUT";
+    break;
+  case 2:
+    manglechainChoice = "FORWARD";
+    break;
+  case 3:
     manglechainChoice = "OUTPUT";
     break;
+  case 4:
+    manglechainChoice = "POSTROUTING";
+    break;
   };
 };
 
@@ -764,11 +1026,17 @@ void knetfilter::mangleactionActivated(int index) {
     mangleactionChoice = "TOS";
     break;
   case 1:
-    mangleactionChoice = "MARK";
+    mangleactionChoice = "DSCP";
     break;
   case 2:
+    mangleactionChoice = "MARK";
+    break;
+  case 3:
     mangleactionChoice = "LOG";
     break;
+  case 4:
+    mangleactionChoice = "ULOG";
+    break;
   };
 };
 
@@ -937,9 +1205,13 @@ void knetfilter::manglepolRules() {
 
       manglepolchain = new QComboBox( FALSE, manglepolList, "Chain");
       manglepolchain->insertItem("PREROUTING");
+      manglepolchain->insertItem("INPUT");
+      manglepolchain->insertItem("FORWARD");
       manglepolchain->insertItem("OUTPUT");
+      manglepolchain->insertItem("POSTROUTING");
       manglepolchain->setFixedHeight(manglepolchain->sizeHint().height());
       connect(manglepolchain,SIGNAL(activated(int)), SLOT(manglepolchainActivated(int))); 
+
       grid->addMultiCellWidget(manglepolchain,0,0,2,3);
 
       QLabel *labelmangleChaindef = new QLabel(manglepolList);
@@ -970,7 +1242,7 @@ void knetfilter::manglepolRules() {
       grid->addWidget(btnClose,6,2);
       
       manglepolList->show();
-      setView(manglepolList);
+      setCentralWidget(manglepolList);
       }
   else
     {
@@ -1034,14 +1306,26 @@ void knetfilter::manglepolchainActivated(int index) {
   switch(index) {
   default:
     manglepolchainChoice = "PREROUTING";
-  case0: 
+  case 0: 
     manglepolchainChoice = "PREROUTING";
     break;
-  case1:
+  case 1:
+    manglepolchainChoice = "INPUT";
+  printf("%s\n",manglepolchainChoice);
+    break;
+  case 2:
+    manglepolchainChoice = "FORWARD";
+  printf("%s\n",manglepolchainChoice);
+    break;
+  case 3:
+    manglepolchainChoice = "OUTPUT";
+  printf("%s\n",manglepolchainChoice);
+    break;
+  case 4:
     manglepolchainChoice = "POSTROUTING";
+  printf("%s\n",manglepolchainChoice);
     break;
   };
-
 };
 
 void knetfilter::manglepolicyActivated(int index) {
