@@ -1,9 +1,9 @@
   /*
 
-  knetfilter v 2.0.0
+  knetfilter v 3.0.2
 
-  Copyright (C) 2000 Luigi Genoni
-  		     venom@DarkStar.sns.it
+  Copyright (C) 2002 Luigi Genoni
+  		     venom@sns.it
   */
   
 
@@ -527,7 +527,7 @@ void knetfilter::masqRules() {
       grid->addWidget(btnClose,9,6);
       
       masqWin->show();
-      setView(masqWin);
+      setCentralWidget(masqWin);
     }
   else
     {
