@@ -1,9 +1,9 @@
   /*
 
-  knetfilter v 2.2.0
+  knetfilter v 3.1.1
 
-  Copyright (C) 2001 Luigi Genoni
-  		     venom@DarkStar.sns.it
+  Copyright (C) 2002 Luigi Genoni
+  		     venom@sns.it
   */
   
 
@@ -45,8 +45,14 @@ void knetfilter::startlistRules() {
 
     rules.clearArguments();
 
-    rules << "iptables" << "-L" << "--line-numbers";
-
+    if ( monitor_resolution == 1 )
+      {
+	rules << "iptables" << "-L" << "--line-numbers" << "--numeric";
+      }
+    else
+      {
+	rules << "iptables" << "-L" << "--line-numbers";
+      }
     bool success=rules.start(KProcess::NotifyOnExit,
 			     KProcess::AllOutput);
     if (success)
@@ -60,8 +66,14 @@ void knetfilter::startmasqEntries() {
 
   entries.clearArguments();
 
-  entries << "iptables" <<"-t" <<"nat" << "-L" << "--line-numbers";
-
+    if ( monitor_resolution == 1 )
+      {
+	entries << "iptables" <<"-t" <<"nat" << "-L" << "--line-numbers" << "--numeric";
+      }
+    else
+      {
+	entries << "iptables" <<"-t" <<"nat" << "-L" << "--line-numbers";
+      }
   bool success=entries.start(KProcess::NotifyOnExit,
   			   KProcess::AllOutput);
   if (success)
@@ -75,8 +87,14 @@ void knetfilter::startmangleEntries() {
 
   entries.clearArguments();
 
-  entries << "iptables" <<"-t" <<"mangle" << "-L" << "--line-numbers";
-
+    if ( monitor_resolution == 1 )
+      {
+	entries << "iptables" <<"-t" <<"mangle" << "-L" << "--line-numbers" << "--numeric";
+      }
+    else
+      {
+	entries << "iptables" <<"-t" <<"mangle" << "-L" << "--line-numbers";
+      }
   bool success=entries.start(KProcess::NotifyOnExit,
   			   KProcess::AllOutput);
   if (success)
@@ -207,7 +225,7 @@ void knetfilter::stopNMap() {
 void knetfilter::polRules() {
   if ( polList == NULL )
     {
-      setupConnect();    
+      //      setupConnect();    
 
       polList = new KNFWidget();
       polList->resize(350,120);
@@ -258,7 +276,7 @@ void knetfilter::polRules() {
       grid->addWidget(btnClose,6,2);
       
       polList->show();
-      setView(polList);
+      setCentralWidget(polList);
     }
   else
     {
@@ -280,23 +298,35 @@ void knetfilter::polRules() {
 void knetfilter::iptablesRules() {
   if ( masqList == NULL )
     {
+      //      setupConnect();
 
       masqList = new KNFWidget();
       masqList->resize(680,420);
       masqList->setCaption("current iptables rules, masqurading and mangle  entries");
 
       QBoxLayout *top = new QBoxLayout(masqList, QBoxLayout::LeftToRight, 7);
-      QGridLayout *grid = new QGridLayout(7,5);
+      QGridLayout *grid = new QGridLayout(8,5);
       top->addLayout(grid,5);
       
       masqBox = new QListBox(masqList);
       grid->addMultiCellWidget(masqBox,0,5,0,4);
 
+      
+      rulesmonitor_resolver= new QCheckBox ( masqList, "Enable reverse resolution" ); 
+      rulesmonitor_resolver->setGeometry(8,8,8,8);
+      rulesmonitor_resolver->setText("Enable reverse resolution");
+      rulesmonitor_resolver->setAutoRepeat( FALSE );
+      rulesmonitor_resolver->setAutoResize( FALSE );
+      connect(rulesmonitor_resolver,SIGNAL(clicked()), SLOT(monitorResolver())); 
+      grid->addMultiCellWidget(rulesmonitor_resolver,6,6,0,2);
+      
+
+
       QPushButton *btnRules = new QPushButton(masqList);
       btnRules->setText("Rules");
       btnRules->setFixedHeight(btnRules->sizeHint().height());
       connect(btnRules,SIGNAL(clicked()), this, SLOT(startlistRules()));
-      grid->addWidget(btnRules,6,0);
+      grid->addWidget(btnRules,7,0);
 
       checkMasquerade();
       if(ip_masquerade==NULL){
@@ -309,23 +339,23 @@ void knetfilter::iptablesRules() {
 	btnTraff->setText("Masq Entries");
 	btnTraff->setFixedHeight(btnTraff->sizeHint().height());
 	connect(btnTraff,SIGNAL(clicked()), this, SLOT(startmasqEntries()));
-	grid->addWidget(btnTraff,6,1);
+	grid->addWidget(btnTraff,7,1);
       }
 
       QPushButton *btnMangle = new QPushButton(masqList);
       btnMangle->setText("Mangle Entries");
       btnMangle->setFixedHeight(btnMangle->sizeHint().height());
       connect(btnMangle,SIGNAL(clicked()), this, SLOT(startmangleEntries()));
-      grid->addWidget(btnMangle,6,2);
+      grid->addWidget(btnMangle,7,2);
   
       QPushButton *btnClose = new QPushButton(masqList);
       btnClose->setText("Close");
       btnClose->setFixedHeight(btnClose->sizeHint().height());
       connect(btnClose,SIGNAL(clicked()), this, SLOT(closeiptablesRules()));
-      grid->addWidget(btnClose,6,4);
+      grid->addWidget(btnClose,7,4);
 
       masqList->show();
-      setView(masqList);
+      setCentralWidget(masqList);
     }
   else
     {
@@ -344,6 +374,7 @@ void knetfilter::iptablesRules() {
 void knetfilter::tcpDump() {
   if ( dumpList == NULL )
     { 
+      //      setupConnect();
 
       dumpList = new KNFWidget();
       dumpList->resize(550,420);
@@ -375,7 +406,7 @@ void knetfilter::tcpDump() {
       grid->addWidget(btnClose,4,3);
   
       dumpList->show();
-      setView(dumpList);
+      setCentralWidget(dumpList);
     }
   else
     {
@@ -395,7 +426,8 @@ void knetfilter::NMap(){
   
   if ( mapList == NULL )
   {
-     
+    //      setupConnect();
+
       mapList = new KNFWidget();
       mapList->resize(640,480);
       mapList->setCaption("nmap");
@@ -474,7 +506,7 @@ void knetfilter::NMap(){
       grid->addWidget(btnClose,8,3);       
       
       mapList->show();
-      setView(mapList); 
+      setCentralWidget(mapList); 
     }
   else
     {
@@ -641,6 +673,7 @@ void knetfilter::listDump(KProcess *, char *data, int len) {
   dumpBox->setTopItem(dumpBox->count()-dumpBox->numItemsVisible());
 };
 
+
 void knetfilter::portDump(KProcess *, char *data, int len) {
 
   if (len<0)
@@ -678,6 +711,18 @@ void knetfilter::portDump(KProcess *, char *data, int len) {
     
 };
   
+
+void knetfilter::monitorResolver() {
+  if (rulesmonitor_resolver->isChecked())
+    {
+      monitor_resolution=0;
+    }
+  else
+    {
+      monitor_resolution=1;
+    }
+};
+
 // probe interface action
 void knetfilter::startprobeInterface() {
 
@@ -1029,6 +1074,53 @@ void knetfilter::flagsActivated(int index) {
   case 1:
     flagChoice = "--syn";
       break;
+  case 2:
+    flagChoice = "-f";
+      break;
+  };
+};
+
+void knetfilter::ecnflagsActivated(int index) {
+  switch(index) {
+  default: 
+    hasEcn = "-v";
+    ecnflagChoice = "-v";
+    ecnParam = "-v";
+  case 0:
+    hasEcn = "-v";
+    ecnflagChoice = "-v";
+    ecnParam = "-v";
+    break;
+  case 1:
+    hasEcn = "ecn";
+    ecnflagChoice = "--ecn-tcp-cwr";
+    ecnParam = "-v";
+    break;
+  case 2:
+    hasEcn = "ecn";
+    ecnflagChoice = "--ecn-tcp-ece";
+    ecnParam = "-v";
+    break;
+  case 3:
+    hasEcn = "ecn";
+    ecnflagChoice = "--ecn-ip-ect";
+    ecnParam = "0";
+    break;
+  case 4:
+    hasEcn = "ecn";
+    ecnflagChoice = "--ecn-ip-ect";
+    ecnParam = "1";
+      break;
+  case 5:
+    hasEcn = "ecn";
+    ecnflagChoice = "--ecn-ip-ect";
+    ecnParam = "2";
+      break;
+  case 6:
+    hasEcn = "ecn";
+    ecnflagChoice = "--ecn-ip-ect";
+    ecnParam = "3";
+      break;
   };
 };
 
