@@ -1,9 +1,9 @@
   /*
 
-  knetfilter v 2.2.0
+  knetfilter v 3.0.2
 
-  Copyright (C) 2001 Luigi Genoni
-  		     venom@DarkStar.sns.it
+  Copyright (C) 2002 Luigi Genoni
+  		     venom@sns.it
   */
   
 
@@ -531,7 +531,7 @@ void knetfilter::natRules() {
       grid->addWidget(btnClose,9,6);
       
       natList->show();
-      setView(natList);
+      setCentralWidget(natList);
     }
   else 
     {
@@ -702,7 +702,7 @@ void knetfilter::natpolRules() {
       grid->addWidget(btnClose,6,2);
       
       natpolList->show();
-      setView(natpolList);
+      setCentralWidget(natpolList);
     }
   else 
     {
